import csv
import json
import sys
import argparse
import csv

from py_jsm_asset.py_jsm_asset import RestAssetCommunicator

parser = argparse.ArgumentParser()


parser.add_argument("-s", "--schema",required = False, help = "the project schema")
parser.add_argument("-t", "--type",required = False, help = "the object type target that contain the object to delete")
parser.add_argument("-i", "--token",required = True, help = "API token file")
parser.add_argument("-a", "--aql",required = True, help = "AQL query to match object to dump")
parser.add_argument("-d", "--dump",required = False, help = "comma separated list of attribute to dump: att1,att2,")
parser.add_argument("-c", "--csv",required = False, help = "specify CSV file for dump, activated with -d option")
parser.add_argument("-r", "--raw",required = False,action='store_true', help = "dump all json (result of the search)")

parser.add_argument("-j", "--json",required = False, help = "specify JSON file for dump")
parser.add_argument("-v", "--verbose",required = False, default=False,action='store_true',help = "verbose")

args = parser.parse_args()

if __name__ == "__main__":

    if args.schema and args.token:
        try:
            file = open(args.token)
        except FileNotFoundError as e:
            print(f"the file \"{args.token}\" can't be found")
            raise FileNotFoundError(e)
        fileText = file.readlines()
        file.close()
        asset=RestAssetCommunicator("https://servicedesk.infn.it/rest/",fileText[0],"1.0",args.schema)
        cnt=0
        fjson=None
        csv_writer=None
        attrs_dump=None
        if args.dump:
            attrs_dump=args.dump.split(',')
        if args.json:
            fjson = open(args.json,"w")
            print(f"writing on \"{args.json}\"...")
        if args.csv and attrs_dump:
            fcsv = open(args.csv,"w")
            print(f"writing on \"{args.csv}\"...")
            head=['Label']
            head.extend(attrs_dump)
            csv_writer = csv.DictWriter(fcsv, fieldnames=head)
            csv_writer.writeheader()
            
        print(f"Searching for \"{args.aql}\" objects...")

        aql=asset.searchObjects(args.aql,args.type,includeAttributes=True)
        lista=aql['objectEntries']
        print(f"Found {len(lista)} objects")
       
        json_out=[]
        csv_out=[]
        for ele in lista:
            dump={}
            csv_dump={}
            dump['name']=ele['label']
            dump['attrs']=[]
            csv_dump['Label']=ele['label']
            for att in ele['attributes']:
                name=att['objectTypeAttribute']['name']
                if attrs_dump:
                    if name not in attrs_dump:
                        continue
                value=""
                if len(att['objectAttributeValues'])>1:
                    value=[]
                    for v in att['objectAttributeValues']:
                        value.append(v['displayValue'])
                else:
                    value= att['objectAttributeValues'][0]['displayValue']
                dump['attrs'].append({'name':name,'value':value});
                csv_dump[name]=value
            if args.raw:
                json_out.append(ele)
            else:
                json_out.append(dump)
            csv_out.append(csv_dump)

        if fjson:
            fjson.write(json.dumps(json_out))
            fjson.close()
        if csv_writer:
            csv_writer.writerows(csv_out)
    
        sys.exit(0)
            



