#!python3.9
import re
import sys
import argparse
import csv

from py_jsm_asset.py_jsm_asset import RestAssetCommunicator

parser = argparse.ArgumentParser()

## Note the object must have the following "text"
## "hostname", "mac", "ip", "router","serverip","nserver","filename"

parser.add_argument("-st", "--type",required = True, help = "source type to be referenced")
parser.add_argument("-dt", "--dest",required = True, help = "destination type where to link")
parser.add_argument("-l", "--link",required = True, help = "destination attribute where to put the reference")

parser.add_argument("-a", "--attribute",required = False, help = "attribute to trigger the link on equality (must be the same name on source and destination)")
parser.add_argument("-s", "--schema",required = True, help = "the project schema")
parser.add_argument("-i", "--token",required = True, help = "API token file")
parser.add_argument("-qs", "--aql_src",required = False, help = "Filter SOURCE AQL query")
parser.add_argument("-qd", "--aql_dst",required = False, help = "Filter DEST AQL query")

parser.add_argument("-v", "--verbose",required = False, default=False,action='store_true',help = "verbose")


args = parser.parse_args()

if __name__ == "__main__":
    fileText=None
    try:
        file = open(args.token)
        fileText = file.readlines()

    except FileNotFoundError as e:
        print(f"the file \"{args.token}\" can't be found")
        raise FileNotFoundError(e)
    asset=RestAssetCommunicator("https://servicedesk.infn.it/rest/",fileText[0],"1.0",args.schema)
#    print(f"writing {len(list(csv_reader))} to schema {args.schema} into {args.type}")
    cnt=0
    query_src=""
    if args.aql_src:
        query_src=args.aql_src
    query_dst=""
    if args.aql_dst:
        query_dst=args.aql_dst
    try:
        print(f"Link \"{args.type}\" -> \"{args.dest}/{args.link}\" with equal \"{args.attribute}\"")
        print(f"Listing \"{args.type}\" query=\"{query_src}\" ...")

        lista_src=asset.searchObjects(query_src,args.type,includeAttributes=True)['objectEntries']
        print(f"- {args.type} found {len(lista_src)} items")
        print(f"Listing {args.dest} query=\"{query_dst}\" ...")
        lista_dst=asset.searchObjects(query_dst,args.dest,includeAttributes=True)['objectEntries']
        print(f"- {args.dest} found {len(lista_dst)} items")

        line_count = 1
        
#        attrs_dump=None
#        if args.attribute:
#            attrs_dump=args.attribute.split(',')
        for dst in lista_dst:
            for src in lista_src:
                sname=src['name']
                dname=dst['name']
                sid=src['id']
                for sa in src['attributes']:
                    for da in dst['attributes']:
                        san=sa['objectTypeAttribute']['name']
                        dan=da['objectTypeAttribute']['name']
                        if (san == args.attribute) and (san == dan):
                            vs=sa['objectAttributeValues'][0]['value'].lower()
                            vd=da['objectAttributeValues'][0]['value'].lower()
                            if  vs!="" and vs==vd:
                                dst_cpy={}
                                attrfound=None
                                ## find reference to see if attach value
                                for dd in dst['attributes']:
                                    lname=dd['objectTypeAttribute']['name']
                                    if lname ==args.link:
                                        attrfound=dd
                                        attreffound=None
                                        vals=[]
                                        for kk in dd['objectAttributeValues']:
                                            if kk['referencedObject']['id'] == sid:
                                                attreffound=sid
                                            else:
                                                vals.append(kk['referencedObject']['id'])
                                                
                                        if attreffound:
                                            print(f"{line_count} skipping @{attreffound} already in \"{dname}\"")
                                        else:
                                          vals.append(sid)
                                          dst_cpy[args.link]=vals
                                          print(f"{line_count} linking \"{sname}\" to {dname} ")
                                          asset.createObject(args.dest,dst_cpy,dname,id=dst['id'],refid=True)
                                          line_count+=1
                                if not(attrfound):
                                    dst_cpy[args.link]=sid
                                    print(f"{line_count} linking \"{sname}\" to {dname} on \"{args.link}\"")
                                    asset.createObject(args.dest,dst_cpy,dname,id=dst['id'],refid=True)
                                    line_count+=1
                                    

                                          

    except Exception as e:
        print(f"\n### ERROR Link aborted because: {e}")



