#!python3.9
import json
import sys
import re
import argparse
import csv

from py_jsm_asset.py_jsm_asset import RestAssetCommunicator

parser = argparse.ArgumentParser()

## Note the object must have the following "text"
## "hostname", "mac", "ip", "router","serverip","nserver","filename"

parser.add_argument("-c", "--csv",required = False, help = "specify CSV file for dump")
parser.add_argument("-j", "--conf",required = False, help = "Configuration File")
parser.add_argument("-d", "--delimiter",required = False, help = "specify field separator")
parser.add_argument("-dr", "--dryrun",required = False, default=False,action='store_true', help = "dry run do not perform insertion")

parser.add_argument("-s", "--schema",required = False, help = "the project schema")
parser.add_argument("-t", "--type",required = False, help = "the object type target fill with entries")
parser.add_argument("-i", "--token",required = False, help = "API token file")
parser.add_argument("-a", "--attribute",required = False, help = "limits the import to a comma separated list of attribute to insert: att1,att2,")
parser.add_argument("-f", "--force",required = False, default=False,help = "continue in case of error")

parser.add_argument("-v", "--verbose",required = False, default=False,action='store_true',help = "verbose")


args = parser.parse_args()

if __name__ == "__main__":
    configuration={}
    attrs_dump=None
    input_filename=None
    attributeMap=None
    filter=None
    dstType=None
    schema=None
    tokenAPI=None
    delim=","
    matchgen=None
    autoincrement=None
    chain=None
    appendgen=None
    counters={}
    if args.conf:
        with open(args.conf, mode='r') as map_file:
            configuration=json.load(map_file)
            if 'mapping' in configuration:
                attributeMap=configuration['mapping']
            if 'regexp' in configuration:
                filter=configuration['regexp']
            if 'exists' in configuration:
                filter=configuration['regexp']
            if 'autoincrement' in configuration:
                autoincrement=configuration['autoincrement']
            if 'chain' in configuration:
                chain=configuration['chain']
            if 'src' in configuration:
                input_filename=configuration['src']
            if  'dest' in configuration:
                dstType=configuration['dest']
            if  'schema' in configuration:
                schema=configuration['schema']
            if  'token' in configuration:
                tokenAPI=configuration['token']
            if  'delimiter' in configuration:
                delim=configuration['delimiter']
            if  ('filter' in configuration) and (len(configuration['filter'])>0):
                attrs_dump=configuration['filter']
            if  ('matchgen' in configuration) :
                matchgen=configuration['matchgen']
            if  ('appendgen' in configuration) :
                appendgen=configuration['appendgen']
                
        
    
    if args.csv:
        input_filename = args.csv
    if args.type:
        dstType=args.type
    if args.schema:
        schema=args.schema
    if args.delimiter:
        delim=args.delimiter
        
    if args.token:
        try:
            file = open(args.token)
            fileText = file.readlines()
            tokenAPI=fileText[0]
            file.close()
        except FileNotFoundError as e:
            print(f"the file \"{args.token}\" can't be found")
            raise FileNotFoundError(e)
        
    if args.attribute:
        attrs_dump=args.attribute.split(',') 
            
    if not(input_filename):
        print("## Error: a CSV filename must be provided")
        sys.exit(1)
    if not(schema):
        print("## Error: a schema must be provided")
        sys.exit(2)
    if not(dstType):
        print("## Error: a destination type must be provided")
        sys.exit(3)
    if not(tokenAPI):
        print("## Error: a authorization token must be provided")
        sys.exit(4)
    
    inserted_keys=[]
    if args.dryrun:
        print("*** DRY RUN ***")
        
    with open(input_filename, mode='r') as csv_file:
        print (f"* opening {input_filename}")
        csv_reader = csv.DictReader(csv_file,delimiter=delim)
        fileText=None
        asset=RestAssetCommunicator("https://servicedesk.infn.it/rest/",tokenAPI,"1.0",schema)
    #    print(f"writing {len(list(csv_reader))} to schema {args.schema} into {args.type}")

        cnt=0
        
        print (f"* retrieving object of type \"{dstType}\"...")
        line_count = 0
        lista=[]
        try:
            lista=asset.searchObjects("",dstType,includeAttributes=True)
        except Exception as e:
            print(f"\n### ERROR search of type '{dstType}' is failed : Import aborted because: {e}")
            sys.exit(1)
        
        print (f"* found {len(lista['objectEntries'])} \"{dstType}\" objects")

        prev_row=None
        list_row=list(csv_reader)

        for line_count, row in enumerate(list_row):
            found=None
            remap_row={}
            skiprow=False
            if filter:
                for filt in filter:
                    if not (filt in row):
                        skiprow=True
                        print(f"{line_count} skipping because filter key \"{filt}\" not found")
                        break
                    else:
                        pattern = r''+filter[filt]
                        if not re.match(pattern,row[filt]):
                            skiprow=True
                            # print(f"{line_count} skipping because filtered out key \"{filt}\"=\"{row[filt]}\" dont match \"{filter[filt]}\"")



            if skiprow:
                continue
            
            if attributeMap:
                for k in attributeMap:
                    if k in row:
                        if isinstance(attributeMap[k], list):
                            for i in attributeMap[k]:
                                if i in remap_row:
                                    if remap_row[i] == "":
                                        remap_row[i]=row[k]
                                    else:
                                        print(f"Warning key {i} already present {remap_row[i]}, skipping set of {row[k]}")

                                else:
                                    remap_row[i]=row[k]
                        else:
                            if attributeMap[k]!="":
                                i=attributeMap[k]
                                ## handle cumulative 
                                if i in remap_row:
                                    if remap_row[i] == "":
                                        remap_row[i]=row[k]
                                    else:
                                        print(f"Warning key {i} already present {remap_row[i]}, skipping set of {row[k]}")
                                else:       
                                    remap_row[attributeMap[k]]=row[k]

            else:
                remap_row=row
            if autoincrement:
                for inc in  autoincrement:
                    counter=0
                    append_str=None
                    sep=""
                    if 'separator' in inc:
                        sep=inc['separator']
                    if not 'step' in inc:
                        inc['step']=1
                    if 'relcounter' in inc and inc['relcounter']:
                        if 'src' in inc and inc['src'] != "":
                            source=inc['src']
                            if source in remap_row and remap_row[source]!="":
                                if remap_row[source] in counters:
                                    counter=counters[remap_row[source]]
                                else:
                                    counters[remap_row[source]]=inc['counter']
                                    counter=inc['counter']
                                counters[remap_row[source]]=counters[remap_row[source]]+inc['step']

                                append_str=remap_row[source]
                            else:
                                print(f"%% {line_count} autoincrement failed {source} not found in {remap_row}")
                    else:
                        if inc['dst'] in counters:
                            counter=counters[inc['dst']]
                            counters[inc['dst']]=counters[inc['dst']]+inc['step']
                        else:
                            counters[inc['dst']]=inc['counter']
                            counter=inc['counter']
                        append_str=""
                    if 'digit' in inc:
                            counter = "{:0{width}d}".format(counter, width=inc['digit'])
                    else:
                        counter=str(counter)
                    if append_str:
                        append_str=append_str+sep+counter        
                        remap_row[inc['dst']]=append_str
            if appendgen:
                for rules in appendgen:
                    append=""
                    separator=""
                    if 'separator' in rules:
                        separator=rules['separator']
                    for app in rules['src']:
                        if app in row:
                            if row[app]!="":
                                append=append+row[app]+separator
                    remap_row[rules['dst']]=append  

            if matchgen:
                for rules in  matchgen:
                    if rules['src'] in remap_row:
                        pattern = r'' +rules['match']
                        name=remap_row[rules['src']]
                        match=re.match(pattern,name)
                        if match:
                            name=""

                            if 'then' in rules:
                                ## set this name
                                name=rules['then']
                            else:
                                if 'prefix' in rules:
                                    name=rules['prefix']
                                if match.groups():
                                    sep=""
                                    append_str=""

                                    if 'separator' in rules:
                                        sep=rules['separator']
                                    for k in match.groups():
                                        append_str=sep+k
                                        
                                    name=name+append_str
                                
                                if 'postfix' in rules:
                                    name=name+rules['postfix']
                            if 'counter' in rules and isinstance(rules['counter'],int):
                                counter = str(rules['counter'])
                                if 'digit' in rules:
                                        counter = "{:0{width}d}".format(rules['counter'], width=rules['digit'])

                                name=name+counter
                                rules['counter']=rules['counter']+1
                            remap_row[rules['dst']]=name
                            
            if not 'Name' in remap_row:
                continue 
            if remap_row['Name'] =="":
                print(f"{line_count} skipping because 'Name' is empty")
                continue
            if  remap_row['Name'].lower() in inserted_keys:
                print(f"{line_count} skipping because 'Name' {remap_row['Name']} is already inserted/updated")
                continue
            
            for n in lista['objectEntries']:
                if n['name']==remap_row['Name']:
                    found=n['id']
                    break
            
            inserted_keys.append(remap_row['Name'].lower())

            if chain:
                for c in  chain:
                    if 'prev' in c and prev_row:
                      remap_row[c['dst']]= prev_row[c['prev']]
                    if 'next' in c and prev_row:
                        if line_count + 1 < len(list_row):
                            nrow=list_row[line_count+1]
                            if nrow[c['next']] != "":
                                remap_row[c['dst']]= nrow[c['next']]

                    
            prev_row=row
            
            
            if found:
                print(f"{line_count} updating \"{remap_row['Name']}\" ")

            else:
                print(f"{line_count} inserting \"{remap_row['Name']}\"")
            try:        
                if attrs_dump:
                    newrow={}
                    for k in attrs_dump:
                        if k in remap_row:
                            newrow[k]=remap_row[k]
                    if args.dryrun:
                        print (f"Type:'{dstType}' Name:{row['Name']} Update:{found} value:{newrow}")
                    else:
                        asset.createObject(dstType,newrow,row['Name'],found)
                        
                else:
                    if args.dryrun:
                        print (f"Type:'{dstType}' Name:{remap_row['Name']} Update:{found} value:{remap_row}")
                    else:
                        asset.createObject(dstType,remap_row,remap_row['Name'],found)

            except Exception as e:
                print(f"\n### ERROR at line {line_count}: Import aborted because: {e}")
                if not args.force:
                    sys.exit(1)



