import re
import argparse
import csv

from py_jsm_asset.py_jsm_asset import RestAssetCommunicator

parser = argparse.ArgumentParser()

## Note the object must have the following "text"
## "hostname", "mac", "ip", "router","serverip","nserver","filename"

parser.add_argument("-c", "--dhcp",required = True, help = "the dhcp file")
parser.add_argument("-d", "--csv",required = False, help = "specify CSV file for dump")

parser.add_argument("-s", "--schema",required = False, help = "the project schema")
parser.add_argument("-t", "--type",required = False, help = "the object type target fill with dhcp entries")
parser.add_argument("-i", "--token",required = False, help = "API token file")

def extract_dhcp_info(line):
    dhcp_info = {}

    # Use regular expressions to extract relevant information
    match = re.match(r'\s*host\s+(\S+)\s+{\s*hardware ethernet\s+(\S+)\s*;\s*fixed-address\s+(\S+)\s*;', line)
    if match:
        dhcp_info["Hostname"] = match.group(1)
        dhcp_info["MAC"] = match.group(2)
        dhcp_info["IP"] = match.group(3)
        matcho = re.match(r'.*option\s+routers\s+(\S+)\s*;', line)
        if matcho:
            dhcp_info["Router"]=matcho.group(1)
        else:
           dhcp_info["Router"]=""
        
        matcho = re.match(r'.*server-identifier\s+(\S+)\s*;', line)
        if matcho:
            dhcp_info["Server"]=matcho.group(1)
        else:
           dhcp_info["Server"]=""    
           
        match = re.match(r'.*next-server\s+(\S+)\s*;\s*filename\s+\"(\S+)\"\s*;', line)
        if match:
            dhcp_info["Next Server"]=match.group(1)
            dhcp_info["Filename"]=match.group(2)
        else:
           dhcp_info["Next Server"]= ""
           dhcp_info["Filename"]=""

    return dhcp_info

def read_dhcp_config(filename):
    dhcp_config = []
    with open(filename, "r") as file:
        for line in file:
            dhcp_info = extract_dhcp_info(line)
            if dhcp_info:
                dhcp_config.append(dhcp_info)
    return dhcp_config

def write_to_csv(data, output_file):
    with open(output_file, "w", newline="") as file:
        writer = csv.DictWriter(file, fieldnames=["Hostname", "MAC", "IP", "Router","Server","Next Server","Filename"])
        writer.writeheader()
        writer.writerows(data)


args = parser.parse_args()

if __name__ == "__main__":
    input_filename = args.dhcp  
    output_filename = input_filename + ".csv"

    dhcp_config_data = read_dhcp_config(input_filename)
    print(f"read {len(dhcp_config_data)} entries")
    if args.csv:
        output_filename=args.csv
        write_to_csv(dhcp_config_data, output_filename)
        print(f"written {len(dhcp_config_data)} entries to {output_filename}")
    
    if args.schema and args.type and args.token:
        try:
            file = open(args.token)
        except FileNotFoundError as e:
            print(f"the file \"{args.token}\" can't be found")
            raise FileNotFoundError(e)
        fileText = file.readlines()
        file.close()
        asset=RestAssetCommunicator("https://servicedesk.infn.it/rest/",fileText[0],"1.0",args.schema)
        print(f"writing {len(dhcp_config_data)} to schema \"{args.schema}\" into \"{args.type}\"")
        cnt=0
        lista=asset.searchObjects("",args.type)
        for ele in dhcp_config_data:
            cnt+=1
            name=ele['Hostname']+"-"+ele['MAC']
            found=None
            for n in lista['objectEntries']:
                if n['label']==name:
                    found=n['id']
                    break
            asset.createObject(args.type,ele,name,found)
            if found:
                print(f"updated \"{name}\" {cnt}/{len(dhcp_config_data)}")             

            else:
                print(f"inserted \"{name}\" {cnt}/{len(dhcp_config_data)}")             



