#!python3.9
import json
import argparse

from py_jsm_asset import RestAssetCommunicator

parser = argparse.ArgumentParser()

## Note the object must have the following "text"
## "hostname", "mac", "ip", "router","serverip","nserver","filename"
parser.add_argument("-c", "--conf", required=True, help="Configuration file")

parser.add_argument("-j", "--json", required=False, help="JSON elements to insert")
parser.add_argument("-dr", "--dryrun", required=False, default=False, action='store_true', help="dry run do not perform insertion")
parser.add_argument("-f", "--force", required=False, default=False, help="continue in case of error")
parser.add_argument("-v", "--verbose", required=False, default=False, action='store_true', help="verbose")

args = parser.parse_args()

def insert_objects_from_json(json_file, communicator, asset_type, mapping):
    with open(json_file, 'r') as file:
        data = json.load(file)
        objects = data.get('objectSchema', {}).get('objects', [])
        for obj in objects:
            attr_list = {k: v for k, v in obj.items()}
            
            # Apply remap logic
            for key, value in mapping.items():
                if key in attr_list:
                    for target_attr in value:
                        if 'Name' in attr_list and target_attr == 'Name':
                            continue
                        attr_list[target_attr] = attr_list[key]
            
            if not 'Name' in attr_list:
                print(f"Skipping object: {str(obj)}, missing Name attribute")
                continue


            try:
                communicator.createObject(asset_type, attr_list, attr_list["Name"])
                print(f"Successfully created asset: {obj.get('hostname')}")
            except Exception as e:
                print(f"Failed to create asset: {obj.get('hostname')}, Error: {e}")
                if not args.force:
                    break

if __name__ == "__main__":
    configuration = {}
    attrs_dump = None

    if args.conf:
        with open(args.conf, 'r') as conf_file:
            configuration = json.load(conf_file)

    communicator = RestAssetCommunicator(
        server=configuration.get('server'),
        bearer=configuration.get('token'),
        version=configuration.get('version'),
        prj_name=configuration.get('schema')
    )

    insert_objects_from_json(args.json, communicator, configuration.get('type'), configuration.get('mapping'))